/*=============================================================================
	UnXC_Game.h: Unreal game class.
=============================================================================*/

/*-----------------------------------------------------------------------------
	Unreal game engine.
-----------------------------------------------------------------------------*/
#if _MSC_VER
#pragma pack (push,4)
#endif

#ifndef XC_ENGINE_API
#define XC_ENGINE_API DLL_IMPORT
#endif

#ifndef NAMES_ONLY
#define AUTOGENERATE_NAME(name) extern XC_ENGINE_API FName XC_ENGINE_##name;
#define AUTOGENERATE_FUNCTION(cls,idx,name)
#endif

AUTOGENERATE_NAME(XC_Engine)
AUTOGENERATE_NAME(XC_GameEngine)
AUTOGENERATE_NAME(XC_ConnectionHandler)
AUTOGENERATE_NAME(XC_Engine_Actor)
AUTOGENERATE_NAME(XC_Listen)

#ifndef NAMES_ONLY

//Do remove debugging instructions
//#define CACUS_DEBUG

#include "XC_CoreObj.h"

//These variables should persist, so we can't really afford to let the garbage collector to mess this up

#ifndef DISABLE_COLLISION
	XC_ENGINE_API extern FCollisionCacus*	CollisionCacus;
#endif

extern UBOOL bHandlesAcquired;

typedef void (UObject::*XCFN_Object_Function)(void);
extern XCFN_Object_Function HashObjectFunc;

typedef void (UObject::*XCFN_Object_Function_INT)(INT iParm1);
extern XCFN_Object_Function_INT UnhashObjectFunc;


/*
static void TestInverse()
{
	for ( FLOAT f=0 ; f<33000 ; f = f*1.4f + 0.005f )
	{
		FLOAT inv = _Reciprocal(f);
		FLOAT div = 1.f/f;
		debugf( NAME_Log, TEXT("Test inverse of %f: %f, %f, %f"), f, div, inv, div-inv);
	}
}
*/

#ifndef DISABLE_SCRIPT
class XC_ENGINE_API AXC_Engine_Actor : public AActor
{
	public:
	AActor* PreLoginHooks[12];
	UObject* ConfigModule;

	DECLARE_FUNCTION(NewDynArrayElement);		//10 - hack, version 10
	DECLARE_FUNCTION(NewDynArrayLength);		//640 - version 10
	DECLARE_FUNCTION(NewDynArrayInsert);		//641 - version 10
	DECLARE_FUNCTION(NewDynArrayRemove);		//642 - version 10
	DECLARE_FUNCTION(execAddToPackagesMap);		//1718 - version 11
	DECLARE_FUNCTION(execIsInPackageMap);		//1719 - version 13
//	DECLARE_FUNCTION(execPreLoginNative);		//3550 - internal
	DECLARE_FUNCTION(AdminLoginNative);		//3551 - internal
	DECLARE_FUNCTION(execCollidingActors);		//3552 - version 12
	DECLARE_FUNCTION(NewVirtualFunction);		//EX_VirtualFunction
	DECLARE_FUNCTION(PlayerSpeechNative);		//Direct binding - version 17
	DECLARE_FUNCTION(execPawnActors);			//3540 - version 19
	DECLARE_FUNCTION(execNavigationActors);		//3541 - version 19
	DECLARE_FUNCTION(execInventoryActors);		//3542 - version 19
	DECLARE_FUNCTION(execDynamicActors);		//3553 - version 19
	DECLARE_FUNCTION(execReplaceFunction);		//3560 - version 19
	DECLARE_FUNCTION(execRestoreFunction);		//3561 - version 19
	
	DECLARE_FUNCTION(execGetMapName_XC);
	DECLARE_FUNCTION(execPlayerCanSeeMe_XC);

	DECLARE_FUNCTION(execGetReachSpec);
	DECLARE_FUNCTION(execSetReachSpec);
	DECLARE_FUNCTION(execReachSpecCount);
	DECLARE_FUNCTION(execAddReachSpec);
	DECLARE_FUNCTION(execFindReachSpec);
	DECLARE_FUNCTION(execCompactPathList);
	DECLARE_FUNCTION(execAllReachSpecs);
	DECLARE_FUNCTION(execLockToNavigationChain);

	//TODO:
	/*
	native(1715) final function Actor SpawnAct
	(
		Class<Actor>	SpawnClass,
		vector		SpawnLocation,
		optional rotator	SpawnRotation,
		optional name	ActorName,
		optional Actor	SpawnOwner,
		optional Pawn	SpawnInstigator,
		optional Actor	Template,
		optional bool	bMayColFail
	);
	*/
	NO_DEFAULT_CONSTRUCTOR(AXC_Engine_Actor)
	DECLARE_CLASS(AXC_Engine_Actor,AActor,0|CLASS_Transient,XC_Engine)
};
#endif

//
// Editor entry point
//
class XC_ENGINE_API UXC_EditorLoader : public UObject
{
	public:
	DECLARE_CLASS(UXC_EditorLoader,UObject,0,XC_Engine)
};


//
// The Unreal game engine, subclassed.
//

struct FGameLevelHeader
{
	ULevel* GLevel;
	ULevel* GEntry;
	UPendingLevel* GPendingLevel;
	FURL			LastURL;
	TArray<FString> ServerActors;
	TArray<FString> ServerPackages;
};


#define XCGE_Defaults GetDefault<UXC_GameEngine>()
class XC_ENGINE_API UXC_GameEngine : public UGameEngine
{
	#include "UXC_GameEngine.h"
};



#endif

AUTOGENERATE_FUNCTION(AXC_Engine_Actor,-1,execGetMapName_XC);
AUTOGENERATE_FUNCTION(AXC_Engine_Actor,-1,execPlayerCanSeeMe_XC);

AUTOGENERATE_FUNCTION(AXC_Engine_Actor,-1,execGetReachSpec);
AUTOGENERATE_FUNCTION(AXC_Engine_Actor,-1,execSetReachSpec);
AUTOGENERATE_FUNCTION(AXC_Engine_Actor,-1,execReachSpecCount);
AUTOGENERATE_FUNCTION(AXC_Engine_Actor,-1,execAddReachSpec);
AUTOGENERATE_FUNCTION(AXC_Engine_Actor,-1,execFindReachSpec);
AUTOGENERATE_FUNCTION(AXC_Engine_Actor,-1,execCompactPathList);
AUTOGENERATE_FUNCTION(AXC_Engine_Actor,-1,execAllReachSpecs);
AUTOGENERATE_FUNCTION(AXC_Engine_Actor,-1,execLockToNavigationChain);



#ifndef NAMES_ONLY
#undef AUTOGENERATE_NAME
#undef AUTOGENERATE_FUNCTION
#endif NAMES_ONLY

#if _MSC_VER
#pragma pack (pop)
#endif

/*-----------------------------------------------------------------------------
	The End.
-----------------------------------------------------------------------------*/
